@@ -1,10 +1,40 @@
 #' Create a new layer
 #'
+#' A layer is a combination of data, stat and geom with a potential position
+#' adjustment. Usually layers are created using \code{geom_*} or \code{stat_*}
+#' calls but it can also be created directly using this function.
+#'
 #' @export
 #' @inheritParams geom_point
-#' @param geom,stat,position Geom, stat and position adjustment to use in
-#'   this layer. Can either be the name of a ggproto object, or the object
-#'   itself.
+#' @param mapping Set of aesthetic mappings created by \code{\link{aes}} or
+#'   \code{\link{aes_}}. If specified and \code{inherit.aes = TRUE} (the
+#'   default), it is combined with the default mapping at the top level of the
+#'   plot. You must supply \code{mapping} if there is no plot mapping.
+#' @param data The data to be displayed in this layer. There are three
+#'    options:
+#'
+#'    If \code{NULL}, the default, the data is inherited from the plot
+#'    data as specified in the call to \code{\link{ggplot}}.
+#'
+#'    A \code{data.frame}, or other object, will override the plot
+#'    data. All objects will be fortified to produce a data frame. See
+#'    \code{\link{fortify}} for which variables will be created.
+#'
+#'    A \code{function} will be called with a single argument,
+#'    the plot data. The return value must be a \code{data.frame.}, and
+#'    will be used as the layer data.
+#' @param geom The geometric object to use display the data
+#' @param stat The statistical transformation to use on the data for this
+#'    layer, as a string.
+#' @param position Position adjustment, either as a string, or the result of
+#'  a call to a position adjustment function.
+#' @param show.legend logical. Should this layer be included in the legends?
+#'   \code{NA}, the default, includes if any aesthetics are mapped.
+#'   \code{FALSE} never includes, and \code{TRUE} always includes.
+#' @param inherit.aes If \code{FALSE}, overrides the default aesthetics,
+#'   rather than combining with them. This is most useful for helper functions
+#'   that define both data and aesthetics and shouldn't inherit behaviour from
+#'   the default plot specification, e.g. \code{\link{borders}}.
 #' @param params Additional parameters to the \code{geom} and \code{stat}.
 #' @param subset DEPRECATED. An older way of subsetting the dataset used in a
 #'   layer.
@@ -16,6 +46,13 @@
 #'   layer(geom = "point", stat = "identity", position = "identity",
 #'     params = list(na.rm = FALSE)
 #'   )
+#'
+#' # use a function as data to plot a subset of global data
+#' ggplot(mpg, aes(displ, hwy)) +
+#'   layer(geom = "point", stat = "identity", position = "identity",
+#'     data = head, params = list(na.rm = FALSE)
+#'   )
+#'
 layer <- function(geom = NULL, stat = NULL,
                   data = NULL, mapping = NULL,
                   position = NULL, params = list(),
@@ -51,9 +88,13 @@ layer <- function(geom = NULL, stat = NULL,
   if (is.character(position))
     position <- find_subclass("Position", position)
 
+  # Special case for na.rm parameter needed by all layers
+  if (is.null(params$na.rm)) {
+    params$na.rm <- FALSE
+  }
+
   # Split up params between aesthetics, geom, and stat
   params <- rename_aes(params)
-
   aes_params  <- params[intersect(names(params), geom$aesthetics())]
   geom_params <- params[intersect(names(params), geom$parameters(TRUE))]
   stat_params <- params[intersect(names(params), stat$parameters(TRUE))]
@@ -101,6 +142,20 @@ Layer <- ggproto("Layer", NULL,
     cat(snakeize(class(self$position)[[1]]), "\n")
   },
 
+  layer_data = function(self, plot_data) {
+    if (is.waive(self$data)) {
+      plot_data
+    } else if (is.function(self$data)) {
+      data <- self$data(plot_data)
+      if (!is.data.frame(data)) {
+        stop("Data function must return a data.frame", call. = FALSE)
+      }
+      data
+    } else {
+      self$data
+    }
+  },
+
   compute_aesthetics = function(self, data, plot) {
     # For annotation geoms, it is useful to be able to ignore the default aes
     if (self$inherit.aes) {
@@ -134,7 +189,11 @@ Layer <- ggproto("Layer", NULL,
     n <- nrow(data)
     if (n == 0) {
       # No data, so look at longest evaluated aesthetic
-      n <- max(vapply(evaled, length, integer(1)))
+      if (length(evaled) == 0) {
+        n <- 0
+      } else {
+        n <- max(vapply(evaled, length, integer(1)))
+      }
     }
     check_aesthetics(evaled, n)
 
@@ -144,6 +203,7 @@ Layer <- ggproto("Layer", NULL,
     } else {
       evaled$PANEL <- data$PANEL
     }
+    evaled <- lapply(evaled, unname)
     evaled <- data.frame(evaled, stringsAsFactors = FALSE)
     evaled <- add_group(evaled)
     evaled
